@@ -5,120 +5,85 @@
 
 /* __START_OF_JASPER_LICENSE__
  * 
- * JasPer Software License
+ * JasPer License Version 2.0
  * 
- * IMAGE POWER JPEG-2000 PUBLIC LICENSE
- * ************************************
+ * Copyright (c) 2001-2006 Michael David Adams
+ * Copyright (c) 1999-2000 Image Power, Inc.
+ * Copyright (c) 1999-2000 The University of British Columbia
  * 
- * GRANT:
+ * All rights reserved.
  * 
- * Permission is hereby granted, free of charge, to any person (the "User")
- * obtaining a copy of this software and associated documentation, to deal
- * in the JasPer Software without restriction, including without limitation
- * the right to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the JasPer Software (in source and binary forms),
- * and to permit persons to whom the JasPer Software is furnished to do so,
- * provided further that the License Conditions below are met.
+ * Permission is hereby granted, free of charge, to any person (the
+ * "User") obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  * 
- * License Conditions
- * ******************
+ * 1.  The above copyright notices and this permission notice (which
+ * includes the disclaimer below) shall be included in all copies or
+ * substantial portions of the Software.
  * 
- * A.  Redistributions of source code must retain the above copyright notice,
- * and this list of conditions, and the following disclaimer.
+ * 2.  The name of a copyright holder shall not be used to endorse or
+ * promote products derived from the Software without specific prior
+ * written permission.
  * 
- * B.  Redistributions in binary form must reproduce the above copyright
- * notice, and this list of conditions, and the following disclaimer in
- * the documentation and/or other materials provided with the distribution.
- * 
- * C.  Neither the name of Image Power, Inc. nor any other contributor
- * (including, but not limited to, the University of British Columbia and
- * Michael David Adams) may be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * 
- * D.  User agrees that it shall not commence any action against Image Power,
- * Inc., the University of British Columbia, Michael David Adams, or any
- * other contributors (collectively "Licensors") for infringement of any
- * intellectual property rights ("IPR") held by the User in respect of any
- * technology that User owns or has a right to license or sublicense and
- * which is an element required in order to claim compliance with ISO/IEC
- * 15444-1 (i.e., JPEG-2000 Part 1).  "IPR" means all intellectual property
- * rights worldwide arising under statutory or common law, and whether
- * or not perfected, including, without limitation, all (i) patents and
- * patent applications owned or licensable by User; (ii) rights associated
- * with works of authorship including copyrights, copyright applications,
- * copyright registrations, mask work rights, mask work applications,
- * mask work registrations; (iii) rights relating to the protection of
- * trade secrets and confidential information; (iv) any right analogous
- * to those set forth in subsections (i), (ii), or (iii) and any other
- * proprietary rights relating to intangible property (other than trademark,
- * trade dress, or service mark rights); and (v) divisions, continuations,
- * renewals, reissues and extensions of the foregoing (as and to the extent
- * applicable) now existing, hereafter filed, issued or acquired.
- * 
- * E.  If User commences an infringement action against any Licensor(s) then
- * such Licensor(s) shall have the right to terminate User's license and
- * all sublicenses that have been granted hereunder by User to other parties.
- * 
- * F.  This software is for use only in hardware or software products that
- * are compliant with ISO/IEC 15444-1 (i.e., JPEG-2000 Part 1).  No license
- * or right to this Software is granted for products that do not comply
- * with ISO/IEC 15444-1.  The JPEG-2000 Part 1 standard can be purchased
- * from the ISO.
- * 
- * THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
- * NO USE OF THE JASPER SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER
- * THIS DISCLAIMER.  THE JASPER SOFTWARE IS PROVIDED BY THE LICENSORS AND
- * CONTRIBUTORS UNDER THIS LICENSE ON AN ``AS-IS'' BASIS, WITHOUT WARRANTY
- * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
- * WARRANTIES THAT THE JASPER SOFTWARE IS FREE OF DEFECTS, IS MERCHANTABLE,
- * IS FIT FOR A PARTICULAR PURPOSE OR IS NON-INFRINGING.  THOSE INTENDING
- * TO USE THE JASPER SOFTWARE OR MODIFICATIONS THEREOF FOR USE IN HARDWARE
- * OR SOFTWARE PRODUCTS ARE ADVISED THAT THEIR USE MAY INFRINGE EXISTING
- * PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER INTELLECTUAL PROPERTY RIGHTS.
- * THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE JASPER SOFTWARE
- * IS WITH THE USER.  SHOULD ANY PART OF THE JASPER SOFTWARE PROVE DEFECTIVE
- * IN ANY RESPECT, THE USER (AND NOT THE INITIAL DEVELOPERS, THE UNIVERSITY
- * OF BRITISH COLUMBIA, IMAGE POWER, INC., MICHAEL DAVID ADAMS, OR ANY
- * OTHER CONTRIBUTOR) SHALL ASSUME THE COST OF ANY NECESSARY SERVICING,
- * REPAIR OR CORRECTION.  UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
- * WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
- * INITIAL DEVELOPER, THE UNIVERSITY OF BRITISH COLUMBIA, IMAGE POWER, INC.,
- * MICHAEL DAVID ADAMS, ANY OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF THE
- * JASPER SOFTWARE, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO
- * THE USER OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR
- * CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION,
- * DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
- * MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF
- * SUCH PARTY HAD BEEN INFORMED, OR OUGHT TO HAVE KNOWN, OF THE POSSIBILITY
- * OF SUCH DAMAGES.  THE JASPER SOFTWARE AND UNDERLYING TECHNOLOGY ARE NOT
- * FAULT-TOLERANT AND ARE NOT DESIGNED, MANUFACTURED OR INTENDED FOR USE OR
- * RESALE AS ON-LINE CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING
- * FAIL-SAFE PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES,
- * AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT
- * LIFE SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
- * JASPER SOFTWARE OR UNDERLYING TECHNOLOGY OR PRODUCT COULD LEAD DIRECTLY
- * TO DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE
- * ("HIGH RISK ACTIVITIES").  LICENSOR SPECIFICALLY DISCLAIMS ANY EXPRESS
- * OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.  USER WILL NOT
- * KNOWINGLY USE, DISTRIBUTE OR RESELL THE JASPER SOFTWARE OR UNDERLYING
- * TECHNOLOGY OR PRODUCTS FOR HIGH RISK ACTIVITIES AND WILL ENSURE THAT ITS
- * CUSTOMERS AND END-USERS OF ITS PRODUCTS ARE PROVIDED WITH A COPY OF THE
- * NOTICE SPECIFIED IN THIS SECTION.
+ * THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS
+ * LICENSE.  NO USE OF THE SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER
+ * THIS DISCLAIMER.  THE SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
+ * "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
+ * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
+ * PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.  IN NO
+ * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
+ * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
+ * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  NO ASSURANCES ARE
+ * PROVIDED BY THE COPYRIGHT HOLDERS THAT THE SOFTWARE DOES NOT INFRINGE
+ * THE PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OF ANY OTHER ENTITY.
+ * EACH COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY TO THE USER FOR CLAIMS
+ * BROUGHT BY ANY OTHER ENTITY BASED ON INFRINGEMENT OF INTELLECTUAL
+ * PROPERTY RIGHTS OR OTHERWISE.  AS A CONDITION TO EXERCISING THE RIGHTS
+ * GRANTED HEREUNDER, EACH USER HEREBY ASSUMES SOLE RESPONSIBILITY TO SECURE
+ * ANY OTHER INTELLECTUAL PROPERTY RIGHTS NEEDED, IF ANY.  THE SOFTWARE
+ * IS NOT FAULT-TOLERANT AND IS NOT INTENDED FOR USE IN MISSION-CRITICAL
+ * SYSTEMS, SUCH AS THOSE USED IN THE OPERATION OF NUCLEAR FACILITIES,
+ * AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL
+ * SYSTEMS, DIRECT LIFE SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH
+ * THE FAILURE OF THE SOFTWARE OR SYSTEM COULD LEAD DIRECTLY TO DEATH,
+ * PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH
+ * RISK ACTIVITIES").  THE COPYRIGHT HOLDERS SPECIFICALLY DISCLAIM ANY
+ * EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.
  * 
  * __END_OF_JASPER_LICENSE__
  */
 
+/*!
+ * @file jas_icc.h
+ * @brief ICC Profile
+ */
+
 #ifndef JAS_ICC_H
 #define	JAS_ICC_H
 
+/* The configuration header file should be included first. */
 #include <jasper/jas_config.h>
+
 #include <jasper/jas_types.h>
 #include <jasper/jas_stream.h>
 
+#include <stdio.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*!
+ * @addtogroup module_cm
+ * @{
+ */
+
 /* Profile file signature. */
 #define	JAS_ICC_MAGIC		0x61637370
 
@@ -301,10 +266,10 @@ typedef struct {
 	char *ascdata; /* ASCII invariant description */
 	jas_iccuint32_t uclangcode; /* Unicode language code */
 	jas_iccuint32_t uclen; /* Unicode localizable description count */
-	uchar *ucdata; /* Unicode localizable description */
+	jas_uchar *ucdata; /* Unicode localizable description */
 	jas_iccuint16_t sccode; /* ScriptCode code */
 	jas_iccuint8_t maclen; /* Localizable Macintosh description count */
-	uchar macdata[69]; /* Localizable Macintosh description */
+	jas_uchar macdata[69]; /* Localizable Macintosh description */
 } jas_icctxtdesc_t;
 
 /* Text type. */
@@ -344,18 +309,20 @@ struct jas_iccattrval_s;
 
 typedef struct {
 	void (*destroy)(struct jas_iccattrval_s *);
-	int (*copy)(struct jas_iccattrval_s *, struct jas_iccattrval_s *);
-	int (*input)(struct jas_iccattrval_s *, jas_stream_t *, int);
+	int (*copy)(struct jas_iccattrval_s *, const struct jas_iccattrval_s *);
+	int (*input)(struct jas_iccattrval_s *, jas_stream_t *, unsigned);
+//#ifdef JAS_ENABLE_ENCODER
 	int (*output)(struct jas_iccattrval_s *, jas_stream_t *);
-	int (*getsize)(struct jas_iccattrval_s *);
-	void (*dump)(struct jas_iccattrval_s *, FILE *);
+//#endif
+	unsigned (*getsize)(const struct jas_iccattrval_s *);
+	void (*dump)(const struct jas_iccattrval_s *, FILE *);
 } jas_iccattrvalops_t;
 
 /* Attribute value type (type and value information). */
 typedef struct jas_iccattrval_s {
-	int refcnt; /* reference count */
+	unsigned refcnt; /* reference count */
 	jas_iccsig_t type; /* type */
-	jas_iccattrvalops_t *ops; /* type-dependent operations */
+	const jas_iccattrvalops_t *ops; /* type-dependent operations */
 	union {
 		jas_iccxyz_t xyz;
 		jas_icccurv_t curv;
@@ -392,8 +359,8 @@ typedef struct {
 } jas_iccattr_t;
 
 typedef struct {
-	int numattrs;
-	int maxattrs;
+	unsigned numattrs;
+	unsigned maxattrs;
 	jas_iccattr_t *attrs;
 } jas_iccattrtab_t;
 
@@ -422,32 +389,145 @@ typedef struct {
 	jas_iccattrvalops_t ops;
 } jas_iccattrvalinfo_t;
 
+/*!
+@brief
+Read an ICC profile from a stream.
+*/
+JAS_EXPORT
 jas_iccprof_t *jas_iccprof_load(jas_stream_t *in);
+
+/*!
+@brief
+Write an ICC profile to a stream.
+*/
+JAS_EXPORT
 int jas_iccprof_save(jas_iccprof_t *prof, jas_stream_t *out);
+
+/*!
+@brief
+Destroy an ICC profile.
+*/
+JAS_EXPORT
 void jas_iccprof_destroy(jas_iccprof_t *prof);
-jas_iccattrval_t *jas_iccprof_getattr(jas_iccprof_t *prof,
+
+/*!
+@brief
+Get an attribute of an ICC profile.
+*/
+JAS_ATTRIBUTE_PURE
+JAS_EXPORT
+jas_iccattrval_t *jas_iccprof_getattr(const jas_iccprof_t *prof,
   jas_iccattrname_t name);
+
+/*!
+@brief
+Set an attribute of an ICC profile.
+*/
+JAS_EXPORT
 int jas_iccprof_setattr(jas_iccprof_t *prof, jas_iccattrname_t name,
   jas_iccattrval_t *val);
-void jas_iccprof_dump(jas_iccprof_t *prof, FILE *out);
-jas_iccprof_t *jas_iccprof_copy(jas_iccprof_t *prof);
-int jas_iccprof_gethdr(jas_iccprof_t *prof, jas_icchdr_t *hdr);
-int jas_iccprof_sethdr(jas_iccprof_t *prof, jas_icchdr_t *hdr);
 
+/*!
+@brief
+Dump an ICC profile to a stream in human-readable format
+for debugging purposes.
+*/
+JAS_EXPORT
+void jas_iccprof_dump(const jas_iccprof_t *prof, FILE *out);
+
+/*!
+@brief
+Create a copy of an ICC profile.
+*/
+JAS_EXPORT
+jas_iccprof_t *jas_iccprof_copy(const jas_iccprof_t *prof);
+
+/*!
+@brief
+Get the header for an ICC profile.
+*/
+JAS_EXPORT
+int jas_iccprof_gethdr(const jas_iccprof_t *prof, jas_icchdr_t *hdr);
+
+/*!
+@brief
+Set the header for an ICC profile.
+*/
+JAS_EXPORT
+int jas_iccprof_sethdr(jas_iccprof_t *prof, const jas_icchdr_t *hdr);
+
+/*!
+@brief
+Destroy an ICC profile attribute.
+*/
+JAS_EXPORT
 void jas_iccattrval_destroy(jas_iccattrval_t *attrval);
-void jas_iccattrval_dump(jas_iccattrval_t *attrval, FILE *out);
+
+/*!
+@brief
+*/
+JAS_EXPORT
+void jas_iccattrval_dump(const jas_iccattrval_t *attrval, FILE *out);
+
+/*!
+@brief
+
+TODO/FIXME
+*/
+JAS_EXPORT
 int jas_iccattrval_allowmodify(jas_iccattrval_t **attrval);
+
+/*!
+@brief
+Create a copy of an ICC profile attribute.
+*/
+JAS_EXPORT
 jas_iccattrval_t *jas_iccattrval_clone(jas_iccattrval_t *attrval);
+
+/*!
+@brief
+Create an ICC profile attribute.
+*/
+JAS_EXPORT
 jas_iccattrval_t *jas_iccattrval_create(jas_iccuint32_t type);
 
-void jas_iccattrtab_dump(jas_iccattrtab_t *attrtab, FILE *out);
+/*!
+@brief
+Dump an ICC profile attribute to a stream in human-readable format
+for debugging purposes.
+*/
+JAS_EXPORT
+void jas_iccattrtab_dump(const jas_iccattrtab_t *attrtab, FILE *out);
 
-extern uchar jas_iccprofdata_srgb[];
-extern int jas_iccprofdata_srgblen;
-extern uchar jas_iccprofdata_sgray[];
-extern int jas_iccprofdata_sgraylen;
-jas_iccprof_t *jas_iccprof_createfrombuf(uchar *buf, int len);
-jas_iccprof_t *jas_iccprof_createfromclrspc(int clrspc);
+/*!
+@brief
+Create an ICC profile from a buffer in memory.
+*/
+JAS_EXPORT
+jas_iccprof_t *jas_iccprof_createfrombuf(const jas_uchar *buf, unsigned len);
+
+/*!
+@brief
+Create an ICC profile from a color space.
+*/
+JAS_EXPORT
+jas_iccprof_t *jas_iccprof_createfromclrspc(unsigned clrspc);
+
+JAS_EXPORT
+extern const jas_uchar jas_iccprofdata_srgb[];
+
+JAS_EXPORT
+extern const unsigned jas_iccprofdata_srgblen;
+
+JAS_EXPORT
+extern const jas_uchar jas_iccprofdata_sgray[];
+
+JAS_EXPORT
+extern const unsigned jas_iccprofdata_sgraylen;
+
+/*!
+ * @}
+ */
 
 #ifdef __cplusplus
 }
